package DAO;

import Exceptions.IngredientDAOException;
import POJO.Ingredient;

import java.sql.*;
import java.util.ArrayList;
import java.util.List;

public class IngredientDAO {
    // JDBC URL, username, and password of PostgresSQL server
    private static final String url = "jdbc:postgresql://localhost:5432/pizzeria";
    private static final String user = "gwendalmargelyetu";
    private static final String password = "moi";

    // JDBC variables for opening, closing and managing connection
    protected static Connection connection;
    private static PreparedStatement preparedStatement;
    protected static ResultSet resultSet;

    // SQL queries
    private static final String SELECT_ALL_QUERY = "SELECT * FROM ingredient";
    private static final String INSERT_QUERY = "INSERT INTO ingredient(nom, prix) VALUES (?, ?)";
    private static final String SELECT_BY_ID_QUERY = "SELECT id,nom,prix FROM ingredient WHERE id = ?";
    private static final String SELECT_BY_NAME_QUERY = "SELECT id,nom,prix FROM ingredient WHERE nom = ?";
    private static final String DELETE_QUERY = "DELETE FROM ingredient WHERE id = ?";
    private static final String UPDATE_QUERY = "UPDATE ingredient SET nom = ?, prix = ? WHERE id = ?";

    // Method to establish database connection
    static Connection connect() throws IngredientDAOException {
        try {
            Class.forName("org.postgresql.Driver");
            connection = DriverManager.getConnection(url, user, password);
        } catch (SQLException | ClassNotFoundException e) {
            throw new IngredientDAOException("Error establishing database connection AND " + e.getMessage(), e);
        }
        return connection;
    }

    // Method to close database connection
    public static void close() throws IngredientDAOException {
        try {
            if (resultSet != null) {
                resultSet.close();
            }
            if (preparedStatement != null) {
                preparedStatement.close();
            }
            if (connection != null) {
                connection.close();
            }
        } catch (SQLException e) {
            throw new IngredientDAOException("Error closing database resources AND " + e.getMessage(), e);
        }
    }

    // Method to retrieve all ingredients from the database
    public static List<Ingredient> getAllIngredients() throws IngredientDAOException {
        List<Ingredient> ingredients = new ArrayList<>();
        try {
            connect();
            preparedStatement = connection.prepareStatement(SELECT_ALL_QUERY);
            resultSet = preparedStatement.executeQuery();

            while (resultSet.next()) {
                int id = resultSet.getInt("id");
                String name = resultSet.getString("nom");
                double prix = resultSet.getDouble("prix");
                Ingredient ingredient = new Ingredient(id, name, prix);
                ingredients.add(ingredient);
            }
        } catch (SQLException e) {
            throw new IngredientDAOException("Error fetching All ingredients AND " + e.getMessage(), e);
        } finally {
            close();
        }
        return ingredients;
    }

    // Method to insert a new ingredient into the database
// Method to insert a new ingredient into the database
    public static Ingredient insertIngredient(Ingredient ingredient) throws IngredientDAOException {
        try {
            connect();
            if (getIngredient(ingredient.getName()) == null) {
                preparedStatement = connection.prepareStatement(INSERT_QUERY, Statement.RETURN_GENERATED_KEYS);
                preparedStatement.setString(1, ingredient.getName());
                preparedStatement.setDouble(2, ingredient.getPrice());
                preparedStatement.executeUpdate();
                ResultSet generatedKeys = preparedStatement.getGeneratedKeys();
                if (generatedKeys.next()) {
                    ingredient.setId(generatedKeys.getInt(1));
                } else {
                    ingredient.setId(-1);
                }
            } else {
                ingredient.setId(getIngredient(ingredient.getName()).getId());
                updateIngredient(ingredient);
                //just to be sure
                ingredient = getIngredient(ingredient.getId());
            }
        } catch (SQLException e) {
            throw new IngredientDAOException("Error inserting ingredient AND " + e.getMessage(), e);
        } finally {
            // Do not close the connection here, as it needs to remain open for subsequent operations
            // close();
        }
        return ingredient;
    }

    // Method to retrieve an ingredient by its ID
    public static Ingredient getIngredient(int id) throws IngredientDAOException {
        Ingredient ingredient = null;
        try {
            connect();
            preparedStatement = connection.prepareStatement(SELECT_BY_ID_QUERY);
            preparedStatement.setInt(1, id);
            resultSet = preparedStatement.executeQuery();

            if (resultSet.next()) {
                String name = resultSet.getString("nom");
                double prix = resultSet.getDouble("prix");
                ingredient = new Ingredient(id, name, prix);
            }
        } catch (SQLException e) {
            throw new IngredientDAOException("Error fetching ingredient AND " + e.getMessage(), e);
        } finally {
            //close();
        }
        return ingredient;
    }

    //Method to retrieve an ingredient by its name
    public static Ingredient getIngredient(String name) throws IngredientDAOException {
        try {
            connect();
            PreparedStatement statement = connection.prepareStatement(SELECT_BY_NAME_QUERY);
            statement.setString(1, name);
            ResultSet resultSet = statement.executeQuery();

            if (resultSet.next()) {
                int id = resultSet.getInt("id");
                double price = resultSet.getDouble("prix");
                return new Ingredient(id, name, price);
            } else {
                // If the ingredient does not exist, return null
                return null;
            }
        } catch (SQLException e) {
            throw new IngredientDAOException("Error fetching ingredient in byName AND " + e.getMessage(), e);
        } finally {
            //close();
        }
    }

    public static void deleteIngredient(int id) throws IngredientDAOException {
        try {
            connect();
            preparedStatement = connection.prepareStatement(DELETE_QUERY);
            preparedStatement.setInt(1, id);
            preparedStatement.executeUpdate();
        } catch (SQLException e) {
            throw new IngredientDAOException("Error deleting ingredient AND " + e.getMessage(), e);
        } finally {
            //close();
        }
    }

    public static void updateIngredient(Ingredient ingredient) throws IngredientDAOException {
        try {
            connect();
            preparedStatement = connection.prepareStatement(UPDATE_QUERY);
            preparedStatement.setString(1, ingredient.getName());
            preparedStatement.setDouble(2, ingredient.getPrice());
            preparedStatement.setInt(3, ingredient.getId());
            preparedStatement.executeUpdate();
        } catch (SQLException e) {
            throw new IngredientDAOException("Error updating ingredient AND " + e.getMessage(), e);
        } finally {
            close();
        }
    }
}
