package DAO;

import Exceptions.IngredientDAOException;
import POJO.Ingredient;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.List;

import static org.junit.jupiter.api.Assertions.*;

public class IngredientDAOTest {

    private IngredientDAO dao;
    private Connection connection;

    @BeforeEach
    public void setUp() throws IngredientDAOException {
        dao = new IngredientDAO();
        connection = IngredientDAO.connect();
    }

    @Test
    public void testGetAllIngredients() throws IngredientDAOException {
        List<Ingredient> ingredients = dao.getAllIngredients();
        assertFalse(ingredients.isEmpty());
    }

    @Test
    public void testGetIngredient() throws IngredientDAOException {
        Ingredient ingredient = IngredientDAO.getIngredient(1);
        assertFalse(ingredient.isEmpty());
    }

    @Test
    public void testInsertIngredient() throws IngredientDAOException {
        String name = "momo";
        double price = 1.5;

        Ingredient newIngredient = new Ingredient();
        newIngredient.setName(name);
        newIngredient.setPrice(price);

        Ingredient savedIngredient = IngredientDAO.insertIngredient(newIngredient);
        assertTrue(savedIngredient.getId() > 0);
        Ingredient retrievedIngredient = IngredientDAO.getIngredient(savedIngredient.getId());
        assertEquals(name, retrievedIngredient.getName());
        assertEquals(price, retrievedIngredient.getPrice());
    }

    @Test
    public void testDeleteIngredient() throws IngredientDAOException {
        IngredientDAO.deleteIngredient(20);
        Ingredient deletedIngredient = IngredientDAO.getIngredient(20);
        assertNull(deletedIngredient);
    }

    @Test
    public void testConnect() throws IngredientDAOException {
        assertNotNull(IngredientDAO.connect());
    }

    @AfterEach
    public void tearDown() throws SQLException, IngredientDAOException {
        IngredientDAO.close();
    }
}

